#include <Windows.h>
#include <stdio.h> 
#include "CondMgre.h"
#include "hsapi.h"

// DESCRIPTION: 	
//	This file implements automatic install of HotSync(R) technology
//	conduits to a target system. This is the main program and implements the
//	following:
//
//		Usage:	ConduitInstall [/o] [/u] arguments
//				where <arguments> is a space-separated list of:
//					<creator>		- creator id
//					<conduit_name>	- display name of conduit
//					<data_dir>		- conduit data directory
//					<data_filename> - conduit data filename
//					<priority>		- conduit priority (0-4)
//					<classname>		- conduit class name;
//					<classpath>		- conduit class path
//					<VM>			- Java VM (SUN or MS)
//
//		When the /u option is specified, only the and <creator> argument is required.
//
//		The files ConduitInstall.exe and Install.exe built by this project should be shipped with your installer
//		and located in the same directory to run properly. 
//			
//			The conduit should be placed in the following directory by your installer: "C:\Program Files\MyDirectory".
//			The CondMgr.dll and HSAPI.dll files should be placed in the following temporary directory by your
//			installer: "C:\Windows\Temp".
//
//			PLEASE NOTE: YOUR INSTALLER SHOULD REMOVE THE TEMPORARY CONDMGR.DLL AND HSAP.>DLL FILES INSTALLED
//			AFTER COMPLETION. WITH MOST INSTALLATION PACKAGES, THESE FILES ARE INCLUDED AS TEMPORARY INSTALLATION
//			FILES. THE INSTALLER USUALLY PLACES THESE IN A TEMPORARY DIRECTORY CHOSEN BY THE INSTALLER. YOU THEN
//			PASS THIS VALUE TO PCI USING An INSTALLER KEYWORD. FOR INSTALLSHIELD, THE KEYWORD IS <SUPPORT_DIR>.
//			THE INSTALLER WILL CLEAN UP THESE TEMPORARY FILES FOR YOU.

#define NUMBER_OF_PARAMS_INSTALL		(9)
#define NUMBER_OF_PARAMS_UNINSTALL		(2)

void usage(char *msg)
{
	// Display usage dialog.
	printf(msg);
	printf("\n\n");
	printf("usage: install [/uoh] <arguments>\n\n");
	printf("\t/u\tUninstall conduit\n");
	printf("\t/o\tOverwrite conduit\n");
	printf("\t/r\tRestart HotSync Manager\n");
	printf("where <arguments> is a space-separated list of:\n");
	printf("\t1. <creator>\t- creator id\n");
	printf("\t2. <title>\t- display name of conduit\n");
	printf("\t3. <data_dir>\t- conduit data directory\n");
	printf("\t4. <data_filename>\t- conduit data filename\n");
	printf("\t5. <priority>\t- conduit priority (0-4)\n");
	printf("\t6. <classname>\t- conduit class name\n");
	printf("\t7. <classpath>\t- conduit class path\n");
	printf("\t8. <VM>\t- Java VM (SUN or MS)\n");
	exit(-1);
}

int installConduit(char *creator, char *title, char *directory, char *file, 
			int priority, char *className, char *classPath, char *VM) 
{
	int err;

	err = CmInstallCreator(creator, CONDUIT_APPLICATION);
	if (err == 0)
	err = CmSetCreatorName(creator, "jsync13.dll");
	if (err == 0)
	err = CmSetCreatorTitle(creator, title);
	if (err == 0)
	err = CmSetCreatorDirectory(creator, directory);
	if (err == 0)
	err = CmSetCreatorFile(creator, file);
	if (err == 0)
	err = CmSetCreatorPriority(creator, priority);
	if (err == 0)
	err = CmSetCreatorValueString(creator, "ClassName", className);
	if (err == 0)
	err = CmSetCreatorValueString(creator, "ClassPath13", classPath); 	    
	if (err == 0)
	err = CmSetCreatorValueString(creator, "VM", VM);
   return err;
}

int uninstallConduit(char *creator)
{
   int numConduitsRemoved = CmRemoveConduitByCreatorID(creator);
	return numConduitsRemoved;
}

int main(int argc, char **argv)
{
	int		result=0;
	char	szMessage[512];
	char	overwrite = FALSE;
	char	uninstall = FALSE;
	char	restartHotsync = FALSE;

	char	*creator;
	char	*title;
	char	*directory;
	char	*file;
	int		priority;
	char	*className;
	char	*classPath;
	char	*VM;
	int	i;

	char *arguments[20];

	// Check for minimum number of arguments
	if( argc < NUMBER_OF_PARAMS_UNINSTALL )
	{
		sprintf( szMessage, "wrong number of parameters: %d", argc-1 );
		usage( szMessage );
	}

	for(i=1; i<argc; i++) {
	    arguments[i] = (char *) malloc(strlen(argv[i])+1);
	    if(*(argv[i])=='\"') {
		strcpy(arguments[i], argv[i]+1);
		arguments[i][strlen(arguments[i])-1] = '\0';
	    } else {
		strcpy(arguments[i], argv[i]);
	    }
	}


	// Check for any command line switches
	while( *(arguments[1]) == '/' )
	{
		int i;
		if( strcmp(arguments[1], "/o") == 0 )
			overwrite = TRUE;
		else if( strcmp(arguments[1], "/u") == 0 )
			uninstall = TRUE;
		else if( strcmp(arguments[1], "/r") == 0 )
			restartHotsync = TRUE;

		--argc;
		for(i=1; i<argc; i++ )
			arguments[i] = arguments[i+1];
	}

	if( !uninstall )
	{
			printf("Param 1: %s\n", arguments[1]);
			printf("Param 2: %s\n", arguments[2]);
			printf("Param 3: %s\n", arguments[3]);
			printf("Param 4: %s\n", arguments[4]);
			printf("Param 5: %s\n", arguments[5]);
			printf("Param 6: %s\n", arguments[6]);
			printf("Param 7: %s\n", arguments[7]);
			printf("Param 8: %s\n", arguments[8]);

		// Check the input parameters
		if( argc != NUMBER_OF_PARAMS_INSTALL )
		{
			sprintf( szMessage, "wrong number of parameters...%d", argc-1 );
//			  usage( szMessage );
			exit(-1);
		}

		creator = arguments[1];
		title = arguments[2];
		directory = arguments[3];
		file = arguments[4];
		// Ensure priority is a digit from 0-4
		if( (strlen(arguments[5]) != 1) || !(isdigit(*(arguments[5]))) )
		{
			sprintf( szMessage, "parameter 5 priority (%s) must be 0-4, single digit", arguments[5] );
			usage( szMessage );
		}
		priority = atoi(arguments[5]);
		className = arguments[6];
		classPath = arguments[7];
		VM = arguments[8];
	}
	else
	{
		// Check the input parameters
		if( argc < NUMBER_OF_PARAMS_UNINSTALL )
		{
			sprintf( szMessage, "wrong number of parameters...%d", argc-1 );
			usage( szMessage );
		}
		creator = arguments[1];
	}
		
	if( uninstall || overwrite )
	{
		// Uninstall the conduit. Returns number of conduits uninstalled.
		result = uninstallConduit(creator);

		// Ignore the case of /o flag and no previous conduit installed
	if (result > 0) {
			result = 0;
		} else {
			if (!overwrite)
				result = 255;
		}

		if (result == 0)
		printf("Uninstall succeeded\n");
		else
		printf("Uninstall failed %d\n", result);
	}

    if (!uninstall) {
	    result = installConduit(creator, title, directory, file,
					priority, className, classPath, VM);

		if (result == 0)
		printf("Registration succeeded\n");
		else
		printf("Registration failed %d\n", result);
	}

    if (restartHotsync && result == 0) {
		printf("Restarting HotSync Manager\n");
		result = HsSetAppStatus(HsRestart, HSFLAG_NONE);
		if (result == 0)
		printf("Restart succeeded\n");
		else
		printf("Restart failed %d\n", result);
    }
/* This works!
    if (result == 0) {
		result = PltInstallFile("Jan-Pascal van Best", "yf145.prc");
		if (result == 0)
		printf("Installation of yanoff succeeded.\n");
		else
		printf("Yanoff installation failed %d\n", result);
    }
*/
//	  printf("Hit the any key to continue\n");
//	  getchar();
	return result;
}

